/*
The following code is

© Andreas Amann, 2006-2011
http://andreasamann.com/

You may incorporate this code into your program(s) without restriction.
This code has been provided "AS IS" and the responsibility for its operation is yours.
You are not permitted to redistribute this code under my name after having made changes.
If you're going to redistribute the code, please make it clear that the code was descended
from my code, but that you've made changes.
*/

// set up our "Widget Common" name space
var WC_ = (function() {
	// private
	var _spinnerAnimation = null;
	
	// public
	return {
		monthNames:
			['Jan','Feb','Mar','Apr','May','Jun','Jul','Aug','Sep','Oct','Nov','Dec'],
		weekdayNames:
			['Sun','Mon','Tue','Wed','Thu','Fri','Sat'],
		getPref: function(prefKey, defaultValue) 
			{
				return window.widget ? ((widget.preferenceForKey(prefKey) != null ? widget.preferenceForKey(prefKey) : defaultValue)) : defaultValue;
			},
		getInstancePref: function(prefKey, defaultValue)
			{
				return window.widget ? (this.getPref(widget.identifier + "-" + prefKey, defaultValue)) : defaultValue;
			},
		setPref: function(prefKey, value)
			{
				if (window.widget)
					widget.setPreferenceForKey(value,prefKey);
			},
		setInstancePref: function(prefKey, value)
			{
				if (window.widget)
					this.setPref(widget.identifier + "-" + prefKey, value);
			},
		animateSpinner: function(elementID, speed)
			{
				speed = speed ? speed : 50;
				if (_spinnerAnimation == null && $(elementID))
				{
					$s(elementID).backgroundPositionX = "0px";
					$s(elementID).visibility = "visible";
					_spinnerAnimation = setInterval("WC_.animateSpinner('" + elementID + "');", speed);
				}
				else if ($(elementID))
					$s(elementID).backgroundPositionX = ($s(elementID).getPropertyValue("background-position-x") == "-160px") ? "0px" : (parseInt($s(elementID).getPropertyValue("background-position")) - 16) + "px";
				else
					this.stopSpinner(elementID);
			},
		stopSpinner: function(elementID)
			{
				if (_spinnerAnimation != null)
				{
					clearInterval(_spinnerAnimation);
					_spinnerAnimation = null;
				}
				if ($(elementID))
					$s(elementID).visibility = "hidden";
			},
		getKeyValue: function(key, plist)
			{
				plist = plist || "Info.plist";
				// read a key from any plist contained in the widget - this allows reading Info.plist entries like the current version for display on the back
				var xml_http = new XMLHttpRequest(); 
				xml_http.open("GET", plist, false); 
				xml_http.send(null); 

				// get all keys and their values - we should always have a "key" followed by its contents
				var entries = $x(xml_http.responseXML,"*"); 
				if (!entries)
					return "";
				var key_value; 

				for (var i = 0; i < entries.length; i++)
				{
					if ((entries[i].tagName.toLowerCase() == "key") && (entries[i].firstChild.data == key))
					{
						if (entries[i+1].tagName.toLowerCase() == "array")
						{
							key_value = new Array();
							var array_values = entries[i+1].childNodes;
							for (var j in array_values)
								if (array_values[j].nodeType == 1)
									key_value.push(array_values[j].firstChild ? array_values[j].firstChild.data : array_values[j].tagName);
						}
						else
							// deal with self-closing entries like <true/>
							key_value = entries[i+1].firstChild ? entries[i+1].firstChild.data : entries[i+1].tagName;
						break; 
					}
				}
				return key_value; 
			},
		visitHomepage: function()
			{
				// add widget name and version information so we can see who actually uses this link…
				var theURL = "http://andreasamann.com/files/DashboardRedirect.html?" + this.getKeyValue("CFBundleDisplayName").replace(/ /g,"_") + "_" + this.getKeyValue("CFBundleShortVersionString");
				if (window.widget)
					widget.openURL(theURL);
				else
					window.open(theURL);
			},
		refreshView: function(theDiv)
			{
				theDiv = theDiv || "dummy";
				// we seem to need this in order to repaint the moved map correctly
				$s(theDiv).visibility = "visible";
				setTimeout("$s('" + theDiv + "').visibility = '';",0);
			},
		setupVerticalScrollbar: function(theScrollbar)
			{
				theScrollbar.trackStartPath = "Common/Scrollbar/vert_track_top.png";
				theScrollbar.trackMiddlePath = "Common/Scrollbar/vert_track_middle.png";
				theScrollbar.trackEndPath = "Common/Scrollbar/vert_track_bottom.png";
				theScrollbar.thumbStartPath = "Common/Scrollbar/vert_scroll_top.png";
				theScrollbar.thumbMiddlePath = "Common/Scrollbar/vert_scroll_middle.png";
				theScrollbar.thumbEndPath = "Common/Scrollbar/vert_scroll_bottom.png";
				theScrollbar.setSize(12);
				theScrollbar.setTrackStart(theScrollbar.trackStartPath, 2);
				theScrollbar.setTrackMiddle(theScrollbar.trackMiddlePath);
				theScrollbar.setTrackEnd(theScrollbar.trackEndPath, 2);
				theScrollbar.setThumbStart(theScrollbar.thumbStartPath, 7);
				theScrollbar.setThumbMiddle(theScrollbar.thumbMiddlePath);
				theScrollbar.setThumbEnd(theScrollbar.thumbEndPath, 7);
			},
		createDateFromTimeString: function(theTime)
			{
				var currentTime = new Date();
				// change times in the form "9:05P" and "9:05PM" to the standard "9:05 PM"
				theTime = theTime.replace(/([0-9]{2}) *([AP])M?$/i,"$1 $2M");
				theTime = this.monthNames[currentTime.getMonth()] + " " + currentTime.getDate() + " " + currentTime.getFullYear() + " " + theTime;
				return (new Date(theTime));
			},
		twoDigNum: function(theNum)
			{
				return ("0" + String(theNum)).substr(String(theNum).length-1);
			},
		ordinalNumber: function(theNum)
			{
				// the following works correctly for any number - passed as string or number
				switch ((parseInt(theNum,10) % 100 < 20 ? parseInt(theNum,10) % 100 : parseInt(theNum,10) % 10))
				{
					case 1:
						return theNum + "st";
					case 2:
						return theNum + "nd";
					case 3:
						return theNum + "rd";
					default:
						return theNum + "th";
				}
			},
		sanitizeString: function(string) {
			// Unescape strings to prevent code injection when setting innerHTML
			// adapted from GMail widget
			// <http://www.google.com/macwidgets/>
				if (!string) return "";
				return unescape(string).replace(/</g, "&lt;").replace(/>/g, "&gt;").replace(/\u0027/g, "&apos;").replace(/\u0022/g, "&quot;");
			},
		html_decode: function(string) {
			var tarea = document.createElement('textarea');
			tarea.innerHTML = string;
			return tarea.value;
			tarea.parentNode.removeChild(tarea);
			}
	}
})();

// return document element
function $(element)
{
	return document.getElementById(element);
}

// return document element's style
function $s(element)
{
	return $(element).style || null;
}

// return XML tags/attributes
function $x(xmlDoc, tagName, attributeName, tagNumber)
{
	if ((!xmlDoc) || (!xmlDoc.getElementsByTagName(tagName).length))
		return null;
	if (attributeName == null)
		return xmlDoc.getElementsByTagName(tagName);
	return xmlDoc.getElementsByTagName(tagName)[tagNumber || 0].getAttribute(attributeName);
}
